------------------------------------------------------
-- HuntersHelper.lua
-- Version 1600.3
------------------------------------------------------

-- Saved configuration & info
FHH_Config = { };
FHH_Config.Tooltip = true;
FHH_NewBeastInfo = { };
FHH_BadBeastInfo = { };

FHH_AbilityInfo = { };
-- Has the following internal structure:
--		REALM_PLAYER = {
--			SKILL
--		}

-- Runtime state
FHH_State = { };
FHH_State.RealmPlayer = nil;
FHH_State.TamingCritter = nil;

-- Constants
MAX_REPORTED_ZONES = 4;
FHH_NotLearnableBy = {
	["bite"] = { "Crab", "Scorpid", "Owl" },
	["claw"] = { "Bat", "Boar", "Crocolisk", "Gorilla", "Hyena", "Spider", "Tallstrider", "Turtle", "Wind Serpent", "Wolf" },
	["dash"] = { "Bat", "Bear", "Carrion Bird", "Crab", "Crocolisk", "Gorilla", "Owl", "Raptor", "Scorpid", "Spider", "Turtle", "Wind Serpent" },
	["dive"] = { "Bear", "Boar", "Cat", "Crab", "Crocolisk", "Gorilla", "Hyena", "Raptor", "Scorpid", "Spider", "Tallstrider", "Turtle", "Wolf"},
};
FHH_RequiredLevel = {
	["growl"] = { 1, 10, 20, 30, 40, 50, 60, },
	["bite"] = { 1, 8, 16, 24, 32, 40, 48, 56, },
	["claw"] = { 1, 8, 16, 24, 32, 40, 48, 56, },
	["cower"] = { 5, 15, 25, 35, 45, 55, },
	["dash"] = { 30, 40, 50, },
	["dive"] = { 30, 40, 50, },
};


function FHH_OnLoad()

	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
	this:RegisterEvent("UNIT_AURA");
	this:RegisterEvent("UNIT_NAME_UPDATE");
	this:RegisterEvent("CRAFT_SHOW");

	-- Register Slash Commands
	SLASH_FHH1 = "/huntershelper";
	SLASH_FHH2 = "/hh";
	SlashCmdList["FHH"] = function(msg)
		FHH_ChatCommandHandler(msg);
	end
	
	GFWUtils.Print(GFW_COLOR_START.."Fizzwidget Hunter's Helper initialized!"..FONT_COLOR_CODE_CLOSE);
	
end

function FHH_OnEvent(event)

	if ( event == "PLAYER_ENTERING_WORLD" ) then
		
		if (FHH_State.RealmPlayer == nil) then
			FHH_State.RealmPlayer = GetCVar("realmName") .. "." .. UnitName("player");
		end
		if (FHH_AbilityInfo[FHH_State.RealmPlayer] == nil or table.getn(FHH_AbilityInfo[FHH_State.RealmPlayer]) == 0) then
			
			local _, myClass = UnitClass("player");
			if (FHH_Config.Tooltip and myClass == "HUNTER" and UnitLevel > 9) then
				FHH_AbilityInfo[FHH_State.RealmPlayer] = { };
				GFWUtils.Print(GFW_COLOR_START.."Hunter's Helper needs to collect info about what pet skills you already know; please open your Beast Training window. (Info on future skills will be collected as they are learned.)"..FONT_COLOR_CODE_CLOSE);
			end
		end

	elseif ( event == "UPDATE_MOUSEOVER_UNIT" ) then
	
		if ( UnitExists("mouseover") and not UnitPlayerControlled("mouseover") and FHH_Config.Tooltip ) then

			local _, myClass = UnitClass("player");
			if (FHH_Config.Tooltip == "hunter" and myClass ~= "HUNTER") then return; end
			
			local creepName = UnitName("mouseover");
			local abilitiesList = nil;
			
			if (FHH_NewBeastInfo[creepName] ~= nil) then
				abilitiesList = GFWTable.Copy(FHH_NewBeastInfo[creepName]);
			elseif (FHH_BeastInfo[creepName] ~= nil) then
				abilitiesList = GFWTable.Copy(FHH_BeastInfo[creepName]);
			end
						
			if (abilitiesList) then
			
				local _, myClass = UnitClass("player");
				if (myClass == "HUNTER" and FHH_AbilityInfo[FHH_State.RealmPlayer] ~= nil and table.getn(FHH_AbilityInfo[FHH_State.RealmPlayer]) > 0) then
					for i=1, table.getn(abilitiesList) do
						if (GFWTable.IndexOf(FHH_AbilityInfo[FHH_State.RealmPlayer], abilitiesList[i]) ~= 0) then
							abilitiesList[i] = GRAY_FONT_COLOR_CODE..abilitiesList[i]..FONT_COLOR_CODE_CLOSE;
						else
							abilitiesList[i] = GREEN_FONT_COLOR_CODE..abilitiesList[i]..FONT_COLOR_CODE_CLOSE;
						end
					end
				end
						
				GameTooltip:AddLine("Abilities: "..table.concat(abilitiesList, ", "), 1.0, 1.0, 1.0);
				GameTooltip:SetHeight(GameTooltip:GetHeight() + 14);
				local width = 20 + getglobal(GameTooltip:GetName().."TextLeft"..GameTooltip:NumLines()):GetWidth();
				if ( GameTooltip:GetWidth() < width ) then
					GameTooltip:SetWidth(width);
				end
			end
		end
		
	elseif ( event == "UNIT_AURA" ) then
	
		if ( arg1 == "player" and FHH_HasTameEffect("player") ) then
			FHH_State.TamingCritter = UnitName("target");
		end
		
	elseif ( event == "UNIT_NAME_UPDATE" ) then
	
		if ( arg1 == "pet" and FHH_State.TamingCritter ) then
			FHH_CheckPetSpells(FHH_State.TamingCritter);
			FHH_State.TamingCritter = nil;
		end
		
	elseif ( event == "CRAFT_SHOW" ) then

		-- Beast Training uses the CraftFrame; we can tell it's not really a craft because it doesn't have a skill-level bar.
		local name, rank, maxRank = GetCraftDisplaySkillLine();
		if ( name ) then return; end
		
		local numCrafts = GetNumCrafts();
		if not ( numCrafts > 0 )then return; end
		
		for i=1, numCrafts do
			local craftName, craftSubSpellName = GetCraftInfo(i);
			_, _, rankNum = string.find(craftSubSpellName, "(%d)");
			local spellInfo = craftName.." "..rankNum;
			if (GFWTable.IndexOf(FHH_AbilityInfo[FHH_State.RealmPlayer], spellInfo) == 0) then
				table.insert(FHH_AbilityInfo[FHH_State.RealmPlayer], spellInfo)
			end;
		end
		table.sort(FHH_AbilityInfo[FHH_State.RealmPlayer]);

	elseif ( event == "CHAT_MSG_SYSTEM" ) then

		_, _, spellName, rankNum = string.find(arg1, "You have learned a new spell: (%w+) %(Rank (%d+)%)");
		DevTools_Dump({spellName, rankNum});
		if (spellName and rankNum) then
			table.insert(FHH_AbilityInfo[FHH_State.RealmPlayer], spellName.." "..rankNum);
			table.sort(FHH_AbilityInfo[FHH_State.RealmPlayer]);
		end
		
	end

end

function FHH_ChatCommandHandler(msg)

	-- Print Help
	if ( msg == "help" ) or ( msg == "" ) then
		GFWUtils.Print(GFW_COLOR_START.."Fizzwidget Hunter's Helper:"..FONT_COLOR_CODE_CLOSE);
		GFWUtils.Print(GFW_COLOR_START.."/huntershelper /hh <command>"..FONT_COLOR_CODE_CLOSE);
		GFWUtils.Print("- "..GFW_COLOR_START.."help"..FONT_COLOR_CODE_CLOSE.." - Print this helplist.");
		GFWUtils.Print("- "..GFW_COLOR_START.."on|off|onyhunter"..FONT_COLOR_CODE_CLOSE.." - Turn display of beast abilities in tooltips on or off, or make them only appear if you're playing a hunter.");
		GFWUtils.Print("- "..GFW_COLOR_START.."status"..FONT_COLOR_CODE_CLOSE.." - Check current settings.");
		GFWUtils.Print("- "..GFW_COLOR_START.."find <ability> <rank>"..FONT_COLOR_CODE_CLOSE.." - List where beasts with a given ability (e.g. Bite 6) can be found.");
		return;
	end
	
	if (msg == "onlyhunter") then
		FHH_Config.Tooltip = "hunter";
		GFWUtils.Print(GFW_COLOR_START.."Hunter's Helper is enabled; beast tooltips will display ability info only when playing a Hunter character."..FONT_COLOR_CODE_CLOSE);
		return;
	end
	if (msg == "on") then
		FHH_Config.Tooltip = true;
		GFWUtils.Print(GFW_COLOR_START.."Hunter's Helper is enabled; beast tooltips will display ability info."..FONT_COLOR_CODE_CLOSE);
		return;
	end
	if (msg == "off") then
		FHH_Config.Tooltip = nil;
		GFWUtils.Print(GFW_COLOR_START.."Hunter's Helper is disabled; no extra info added to tooltips."..FONT_COLOR_CODE_CLOSE);
		return;
	end

	-- Check Status
	if ( msg == "status" ) then
		if ( FHH_Config.Tooltip == "hunter" ) then
			GFWUtils.Print(GFW_COLOR_START.."Hunter's Helper is enabled; beast tooltips will display ability info."..FONT_COLOR_CODE_CLOSE);
		elseif ( FHH_Config.Tooltip ) then
			GFWUtils.Print(GFW_COLOR_START.."Hunter's Helper is enabled; beast tooltips will display ability info."..FONT_COLOR_CODE_CLOSE);
		else
			GFWUtils.Print(GFW_COLOR_START.."Hunter's Helper is disabled; no extra info added to tooltips."..FONT_COLOR_CODE_CLOSE);
		end
		return;
	end
	
	local _, _, cmd, spellName, rankNum = string.find(msg, "(find%w*) (%w+) (%d+)");
	if (cmd) then
		if not (spellName and rankNum) then
			GFWUtils.Print("Usage: "..GFW_COLOR_START.."/hh find <ability> <rank>"..FONT_COLOR_CODE_CLOSE);
			return;
		end
		
		spellName = string.lower(spellName);
		niceSpellName = string.upper(string.sub(spellName, 1, 1))..string.sub(spellName, 2);
		rankNum = tonumber(rankNum);
		if (spellName == "growl") then
			if (rankNum <= 2) then
				GFWUtils.Print(GFW_COLOR_START.."You should already know "..FONT_COLOR_CODE_CLOSE..niceSpellName.." "..rankNum..GFW_COLOR_START.." if you've completed your level 10 Hunter quests."..FONT_COLOR_CODE_CLOSE);
			elseif (rankNum > 2 and rankNum < 8) then
				GFWUtils.Print(niceSpellName.." "..rankNum..GFW_COLOR_START.." is taught by Pet Trainers, who can be found in several towns and most major cities."..FONT_COLOR_CODE_CLOSE);
			else
				GFWUtils.Print(niceSpellName..GFW_COLOR_START.." is not known to have a "..FONT_COLOR_CODE_CLOSE.."Rank "..rankNum..".");
			end
		else
			if not (FHH_SpellInfo[spellName]) then
				GFWUtils.Print(niceSpellName..GFW_COLOR_START.." is not a known beast ability."..FONT_COLOR_CODE_CLOSE);
				return;
			end
			if not (FHH_SpellInfo[spellName][rankNum]) then
				GFWUtils.Print(niceSpellName..GFW_COLOR_START.." is not known to have a "..FONT_COLOR_CODE_CLOSE.."Rank "..rankNum..".");
				return;
			end
		end
		
		-- report minimum pet level for ability
		local petLevel = 60;
		if (UnitExists("pet")) then
			petLevel = tonumber(UnitLevel("pet"));
		end
		local minLevel = FHH_RequiredLevel[spellName][rankNum];
		if (minLevel == nil) then
			GFWUtils.Print(RED_FONT_COLOR_CODE.."Hunter's Helper error: can't find required level for "..niceSpellname.." "..rankNum..". Please report to gazmik@fizzwidget.com"..FONT_COLOR_CODE_CLOSE);
		else
			if (petLevel >= minLevel) then
				GFWUtils.Print(niceSpellName.." "..rankNum..GFW_COLOR_START.." requires pet level "..FONT_COLOR_CODE_CLOSE..minLevel..".");
			else
				GFWUtils.Print(niceSpellName.." "..rankNum..GFW_COLOR_START.." requires pet level "..FONT_COLOR_CODE_CLOSE..RED_FONT_COLOR_CODE..minLevel..FONT_COLOR_CODE_CLOSE..".");
			end
		end
	
		-- report exceptions
		local petFamily = nil;
		if (UnitExists("pet")) then
			petFamily = UnitCreatureFamily("pet");
		end
		local exceptions = FHH_NotLearnableBy[spellName];
		if (exceptions ~= nil) then
			local exceptionsList = table.concat(exceptions, ", ");
			if (petFamily and string.find(exceptionsList, petFamily)) then
				exceptionsList = string.gsub(exceptionsList, petFamily, RED_FONT_COLOR_CODE..petFamily..FONT_COLOR_CODE_CLOSE);
			end
			GFWUtils.Print(niceSpellName..GFW_COLOR_START.." is not learnable by: "..FONT_COLOR_CODE_CLOSE..exceptionsList..".");
		end

		if (spellName == "growl") then return; end

		GFWUtils.Print(niceSpellName.." "..rankNum..GFW_COLOR_START.." can be learned from:"..FONT_COLOR_CODE_CLOSE);
		local numReportedZones = 0;
		local zoneName = GFWZones.NormalizeZoneName(GetRealZoneText());
		if (FHH_SpellInfo[spellName][rankNum][zoneName] ~= nil and table.getn(FHH_SpellInfo[spellName][rankNum][zoneName]) > 0) then
			GFWUtils.Print(GFW_COLOR_START..zoneName..": "..FONT_COLOR_CODE_CLOSE..FHH_CreatureListString(FHH_SpellInfo[spellName][rankNum][zoneName]));
			numReportedZones = numReportedZones + 1;
		end
		local zones = GFWZones.ConnectionsForZone(zoneName);
		for numSteps=1, table.getn(zones) do
			for zoneIndex=1, table.getn(zones[numSteps]) do
				local zoneMobList = FHH_SpellInfo[spellName][rankNum][zones[numSteps][zoneIndex]];
				if (zoneMobList ~= nil and table.getn(zoneMobList) > 0) then
					GFWUtils.Print(GFW_COLOR_START..zones[numSteps][zoneIndex]..": "..FONT_COLOR_CODE_CLOSE..FHH_CreatureListString(zoneMobList));
					numReportedZones = numReportedZones + 1;
					if (numReportedZones >= MAX_REPORTED_ZONES) then return; end
				end
			end
		end
		
		if (numReportedZones == 0) then
			-- if we get here, we think we know about the spell but can't find beasts with it in our table. this shouldn't happen.
			GFWUtils.Print(RED_FONT_COLOR_CODE.."Hunter's Helper error: can't find "..niceSpellname.." "..rankNum.." in lookup table. Please report to gazmik@fizzwidget.com"..FONT_COLOR_CODE_CLOSE);
		end
		return;
	end
	
end

function FHH_CreatureListString(critterList)
	local listString = ""
	for _, name in critterList do
		local info = FHH_BeastLevels[name];
		if (info == nil) then
			listString = listString..", ";
		else
			listString = listString .. name .. " ";
			local myLevel = UnitLevel("player");
			local minLevel = info.min;
			local maxLevel = info.max;
			if (info.min > UnitLevel("player")) then
				minLevel = RED_FONT_COLOR_CODE..info.min..FONT_COLOR_CODE_CLOSE;
			end
			if (info.max > UnitLevel("player")) then
				maxLevel = RED_FONT_COLOR_CODE..info.max..FONT_COLOR_CODE_CLOSE;
			end
			if (info.min == info.max) then			
				listString = listString.."("..minLevel;
			else
				listString = listString.."("..minLevel.."-"..maxLevel;
			end
			if (info.type == nil) then
				listString = listString.."), ";
			else
				listString = listString.." "..info.type.."), ";
			end				
		end
	end
	listString = string.gsub(listString, ", $", "");
	return listString;
end

function FHH_HasTameEffect(unit)

	local i = 1;
	local buff;
	buff = UnitBuff(unit, i);
	while buff do
		if ( string.find(buff, "Ability_Hunter_BeastTaming") ) then
			return true;
		end
		i = i + 1;
		buff = UnitBuff(unit, i);
	end
	return false;

end

function FHH_CheckPetSpells(critter)

	local CurrentPetSpells = { };
	local i = 1;
	local spellName, spellRank = GetSpellName(i, BOOKTYPE_PET);
	while spellName do
		_, _, rankNum = string.find(spellRank, "(%d)");
		table.insert(CurrentPetSpells, spellName.." "..rankNum);
		i = i + 1;
		spellName, spellRank = GetSpellName(i, BOOKTYPE_PET);
	end
	
	if ( table.getn(CurrentPetSpells) == 0 ) then return; end
	
	if ( FHH_BeastInfo[critter] ) then
	
		-- record any spells the critter has that our built-in table doesn't know about 
		local UnknownPetSpells = { };
		for i=1, table.getn(CurrentPetSpells) do
			if ( GFWTable.IndexOf(FHH_BeastInfo[critter], CurrentPetSpells[i]) == 0 ) then
				table.insert(UnknownPetSpells, CurrentPetSpells[i]);
			end
		end
		if ( table.getn(UnknownPetSpells) > 0 ) then
			FHH_NewBeastInfo[critter] = CurrentPetSpells; -- we want to remember the entire current spells list
			local details = "(expected "..table.concat(FHH_BeastInfo[critter], ", ").."; found "..concat(CurrentPetSpells, ", ")..").";
			GFWUtils.Print(GFW_COLOR_START.."Hunter's Helper has incorrect data on "..critter..details.." Please submit a correction to gazmik@fizzwidget.com.)"..FONT_COLOR_CODE_CLOSE);
		end
		
		-- record any spells our built-in table thinks the critter has, but the critter actually doesn't
		local WrongPetSpells = { };
		for i=1, table.getn(FHH_BeastInfo[critter]) do
			if ( GFWTable.IndexOf(CurrentPetSpells, FHH_BeastInfo[critter][i]) == 0 ) then
				table.insert(WrongPetSpells, FHH_BeastInfo[critter][i]);
			end
		end
		if ( table.getn(UnknownPetSpells) > 0 ) then
			FHH_BadBeastInfo[critter] = WrongPetSpells;
			local details = "(expected "..table.concat(FHH_BeastInfo[critter], ", ").."; found "..concat(CurrentPetSpells, ", ")..").";
			GFWUtils.Print(GFW_COLOR_START.."Hunter's Helper has incorrect data on "..critter..details.." Please submit a correction to gazmik@fizzwidget.com.)"..FONT_COLOR_CODE_CLOSE);
		end
		
	else
	
		-- this pet is entirely new to our list
		FHH_NewBeastInfo[critter] = CurrentPetSpells;
		local details = "(found "..concat(CurrentPetSpells, ", ")..").";
		GFWUtils.Print(GFW_COLOR_START.."Hunter's Helper has no data on "..critter..details.." Please submit a correction to gazmik@fizzwidget.com.)"..FONT_COLOR_CODE_CLOSE);

	end
	
end
